# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
from dateutil.parser import parse as parse_date

from . import base
from . import connection


class LXDOperation(base.LXDBase):
    def operation_list(self):
        (state, data) = self.connection.get_object('GET', '/1.0/operations')
        return [operation.split('/1.0/operations/')[-1]
                for operation in data['metadata']]

    def operation_show(self, operation):
        (state, data) = self.connection.get_object('GET', '/1.0/operations/%s'
                                                   % operation)
        return {
            'operation_create_time': self.operation_create_time(operation, data.get('metadata')),
            'operation_update_time': self.operation_update_time(operation, data.get('metadata')),
            'operation_status_code': self.operation_status_code(operation, data.get('metadata'))
        }

    def operation_create_time(self, operation, data):
        if data is None:
            (state, data) = self.connection.get_object('GET', '/1.0/operations/%s'
                                                       % operation)
            data = data.get('metadata')
        return parse_date(data['created_at']).strftime('%Y-%m-%d %H:%M:%S')

    def operation_update_time(self, operation, data):
        if data is None:
            (state, data) = self.connection.get_object('GET', '/1.0/operations/%s'
                                                       % operation)
            data = data.get('metadata')
        return parse_date(data['updated_at']).strftime('%Y-%m-%d %H:%M:%S')

    def operation_status_code(self, operation, data):
        if data is None:
            (state, data) = self.connection.get_object('GET', '/1.0/operations/%s'
                                                       % operation)
            data = data.get('metadata')
        return data['status']

    def operation_wait(self, operation, status_code, timeout):
        return self.connection.get_status('GET', '/1.0/operations/%s/wait?status_code=%s&timeout=%s'
                                          % (operation, status_code, timeout))

    def operation_delete(self, operation):
        return self.connection.get_status('DELETE', '/1.0/operations/%s'
                                          % operation)




# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from . import connection
from . import container
from . import certificate
from . import hosts
from . import image
from . import network
from . import operation
from . import profiles


class API(object):
    def __init__(self, host=None, port=8443):
        conn = self.connection = connection.LXDConnection(host=host, port=port)
        self.hosts = hosts.LXDHost(conn)
        self.image = image.LXDImage(conn)
        self.alias = image.LXDAlias(conn)
        self.network = network.LXDNetwork(conn)
        self.operation = operation.LXDOperation(conn)
        self.profiles = profiles.LXDProfile(conn)
        self.certificate = certificate.LXDCertificate(conn)
        self.container = container.LXDContainer(conn)

    # host
    def host_ping(self):
        return self.hosts.host_ping()

    def host_info(self):
        return self.hosts.host_info()

    def get_lxd_api_compat(self, data=None):
        return self.hosts.get_lxd_api_compat(data)

    def get_lxd_host_trust(self, data=None):
        return self.hosts.get_lxd_host_trust(data)

    def get_lxd_backing_fs(self, data=None):
        return self.hosts.get_lxd_backing_fs(data)

    def get_lxd_driver(self, data=None):
        return self.hosts.get_lxd_driver(data)

    def get_lxc_version(self, data=None):
        return self.hosts.get_lxc_version(data)

    def get_lxd_version(self, data=None):
        return self.hosts.get_lxd_version(data)

    def get_kernel_version(self, data=None):
        return self.hosts.get_kernel_version(data)

    # images
    def image_list(self):
        return self.image.image_list()

    def image_defined(self, image):
        return self.image.image_defined(image)

    def image_search(self, params):
        return self.image.image_list_by_key(params)

    def image_info(self, image):
        return self.image.image_info(image)

    def image_upload_date(self, image, data=None):
        return self.image.get_image_date(image, data, 'uploaded_at')

    def image_create_date(self, image, data=None):
        return self.image.get_image_date(image, data, 'created_at')

    def image_expire_date(self, image, data=None):
        return self.image.get_image_date(image, data, 'expires_at')

    def image_upload(self, path=None, data=None):
        return self.image.image_upload(path=path, data=data)

    def image_delete(self, image):
        return self.image.image_delete(image)

    def image_export(self, image):
        return self.image.image_export(image)

    def image_update(self, image):
        return self.image.image_update(image)

    def image_rename(self, image):
        return self.image.image_rename(image)

    # alias
    def alias_list(self):
        return self.alias.alias_list()

    def alias_defined(self, alias):
        return self.alias.alias_defined(alias)

    def alias_create(self, alias):
        return self.alias.alias_create(alias)

    def alias_update(self, alias):
        return self.alias.alias_update(alias)

    def alias_show(self, alias):
        return self.alias.alias_show(alias)

    def alias_rename(self, alias):
        return self.alias.alias_rename(alias)

    def alias_delete(self, alias):
        return self.alias.alias_delete(alias)

    # containers:
    def container_list(self):
        return self.container.container_list()

    def container_defined(self, container):
        return self.container.container_defined(container)

    def container_running(self, container):
        return self.container.container_running(container)

    def container_init(self, container):
        return self.container.container_init(container)

    def container_update(self, container, config):
        return self.container.container_update(container, config)

    def container_state(self, container):
        return self.container.container_state(container)

    def container_start(self, container, timeout):
        return self.container.container_start(container, timeout)

    def container_stop(self, container, timeout):
        return self.container.container_stop(container, timeout)

    def container_suspend(self, container, timeout):
        return self.container.container_suspend(container, timeout)

    def container_resume(self, container, timeout):
        return self.container.container_resume(container, timeout)

    def container_reboot(self, container, timeout):
        return self.container.container_reboot(container, timeout)

    def container_destroy(self, container):
        return self.container.container_destroy(container)

    def get_container_log(self, container):
        return self.container.get_container_log(container)

    # file operations
    def get_container_file(self, container, filename):
        return self.container.get_container_file(container, filename)

    def container_publish(self, container):
        return self.container.container_publish(container)

    def put_container_file(self):
        raise NotImplemented()

    # snapshots
    def container_snapshot_list(self, container):
        return self.container.snapshot_list(container)

    def container_snapshot_create(self, container, config):
        return self.container.snapshot_create(container, config)

    def container_snapshot_info(self, container, snapshot):
        return self.container.snapshot_info(container, snapshot)

    def container_snapshot_reanme(self, container, snapshot, config):
        return self.container.snapshot_rename(container, snapshot, config)

    def container_snaphsot_delete(self, container, snapshot):
        return self.container.snapshot_delete(container, snapshot)

    # misc container
    def container_run_command(self, container, args, interactive=False,
                              web_sockets=False, env=None):
        return self.container.run_command(container, args, interactive,
                                          web_sockets, env)

    # certificates
    def certificate_list(self):
        return self.certificate.certificate_list()

    def certificate_show(self, fingerprint):
        return self.certificate.certificate_show(fingerprint)

    def certificate_delete(self, fingerprint):
        return self.certificate.certificate_delete(fingerprint)

    def certificate_create(self, fingerprint):
        return self.certificate.certificate_create(fingerprint)

    # profiles
    def profile_create(self, profile):
        ''' Create LXD profile '''
        return self.profiles.profile_create(profile)

    def profile_show(self, profile):
        ''' Show LXD profile '''
        return self.profiles.profile_show(profile)

    def profile_defined(self, profile):
        ''' Check to see if profile is defined. '''
        return self.profiles.profile_defined(profile)

    def profile_list(self):
        ''' List LXD profiles '''
        return self.profiles.profile_list()

    def profile_update(self, profile, config):
        ''' Update LXD profile '''
        return self.profiles.profile_update(profile, config)

    def profile_rename(self, profile, config):
        ''' Rename LXD profile '''
        raise NotImplemented()

    def profile_delete(self, profile):
        ''' Delete LXD profile '''
        return self.profiles.profile_delete(profile)

    # lxd operations
    def list_operations(self):
        return self.operation.operation_list()

    def wait_container_operation(self, operation, status_code, timeout):
        return self.operation.operation_wait(operation, status_code, timeout)

    def operation_delete(self, operation):
        return self.operation.operation_delete(operation)

    def operation_info(self, operation):
        return self.operation.operation_show(operation)

    def operation_show_create_time(self, operation, data=None):
        return self.operation.operation_create_time(operation, data)

    def operation_show_update_time(self, operation, data=None):
        return self.operation.operation_update_time(operation, data)

    def operation_show_status(self, operation, data=None):
        return self.operation.operation_status_code(operation, data)

    # networks
    def network_list(self):
        return self.network.network_list()

    def network_show(self, network):
        return self.network.network_show(network)

    def network_show_name(self, network, data=None):
        return self.network.show_network_name(network, data)

    def network_show_type(self, network, data=None):
        return self.network.show_network_type(network, data)

    def network_show_members(self, network, data=None):
        return self.network.show_network_members(network, data)



# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import httplib
import json
import os
import socket
import ssl

from . import utils


class UnixHTTPConnection(httplib.HTTPConnection):
    def __init__(self, path, host='localhost', port=None, strict=None,
                 timeout=None):
        httplib.HTTPConnection.__init__(self, host, port=port,
                                        strict=strict,
                                        timeout=timeout)

        self.path = path

    def connect(self):
        sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        sock.connect(self.path)
        self.sock = sock


class HTTPSConnection(httplib.HTTPConnection):
    default_port = 8443

    def __init__(self, *args, **kwargs):
        httplib.HTTPConnection.__init__(self, *args, **kwargs)

    def connect(self):
        sock = socket.create_connection((self.host, self.port),
                                        self.timeout, self.source_address)
        if self._tunnel_host:
            self.sock = sock
            self._tunnel()

        (cert_file, key_file) = self._get_ssl_certs()
        self.sock = ssl.wrap_socket(sock, certfile=cert_file,
                                    keyfile=key_file)

    @staticmethod
    def _get_ssl_certs():
        return (os.path.join(os.environ['HOME'], '.config/lxc/client.crt'),
                os.path.join(os.environ['HOME'], '.config/lxc/client.key'))


class LXDConnection(object):
    def __init__(self, host=None, port=8443):
        if host:
            self.host = host
            self.port = port
            self.unix_socket = None
        else:
            if 'LXD_DIR' in os.environ:
                self.unix_socket = os.path.join(os.environ['LXD_DIR'], 'unix.socket')
            else:
                self.unix_socket = '/var/lib/lxd/unix.socket'
            self.host, self.port = None, None
        self.connection = None

    def get_connection(self):
        if self.host:
            return HTTPSConnection(self.host, self.port)
        return UnixHTTPConnection(self.unix_socket)

    def get_object(self, *args, **kwargs):
        self.connection = self.get_connection()
        self.connection.request(*args, **kwargs)
        response = self.connection.getresponse()
        state = response.status
        data = json.loads(response.read())
        if not data:
            msg = "Null Data"
            raise Exception(msg)
        elif state == 200 or (state == 202 and data.get('status_code') == 100):
            return state, data
        else:
            utils.get_lxd_error(state, data)

    def get_status(self, *args, **kwargs):
        status = False
        self.connection = self.get_connection()
        self.connection.request(*args, **kwargs)
        response = self.connection.getresponse()
        state = response.status
        data = json.loads(response.read())
        if not data:
            msg = "Null Data"
            raise Exception(msg)
        elif data.get('error'):
            utils.get_lxd_error(state, data)
        elif state == 200 or (state == 202 and data.get('status_code') == 100):
            status = True
        return status

    def get_raw(self, *args, **kwargs):
        self.connection = self.get_connection()
        self.connection.request(*args, **kwargs)
        response = self.connection.getresponse()
        body = response.read()
        if not body:
            msg = "Null Body"
            raise Exception(msg)
        elif response.status == 200:
            return body
        else:
            msg = "Failed to get raw response"
            raise Exception(msg)



# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
import unittest

from pylxd import api
from pylxd import connection

import fake_api

class LXDUnitTestAlias(unittest.TestCase):
    def setUp(self):
        super(LXDUnitTestAlias, self).setUp()
        self.lxd = api.API()

    def test_alias_list(self):
        with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', fake_api.fake_alias_list())
            self.assertEqual(1, len(self.lxd.alias_list()))



# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os

from . import base
from . import connection

class LXDContainer(base.LXDBase):
    # containers:
    def container_list(self):
        (state, data) = self.connection.get_object('GET', '/1.0/containers')
        return [container.split('/1.0/containers/')[-1] for container in data['metadata']]

    def container_defined(self, container):
        (state, data) = self.connection.get_object('GET', '/1.0/containers/%s/state'
                                                % container)
        return data.get('status')

    def container_running(self, container):
        (state, data) = self.connection.get_object('GET',
                                                  '/1.0/containers/%s/state'
                                                  % container)
        data = data.get('metadata')
        container_running = False
        if data['status'] in ['RUNNING', 'STARTING', 'FREEZING,FROZEN',
                                'THAWED']:
           container_running = True
        return container_running


    def container_init(self, container):
        return self.connection.get_object('POST', '/1.0/containers',
                                          json.dumps(container))

    def container_update(self, container, config):
        return self.connection.get_object('PUT', '/1.0/containers/%s'
                                          % container, json.dumps(config))

    def container_defined(self, container):
        return self.connection.get_status('GET', '/1.0/containers/%s/state'
                                          % container)


    def container_state(self, container):
        (state, data) = self.connection.get_object('GET', '/1.0/containers/%s/state'
                                                   % container)
        return data['metadata']['status']


    def container_start(self, container, timeout):
        action = {'action': 'start', 'timeout': timeout}
        return self.connection.get_object('PUT', '/1.0/containers/%s/state'
                                          % container,
                                          json.dumps(action))

    def container_stop(self, container, timeout):
        action = {'action': 'stop', 'timeout': timeout}
        return self.connection.get_object('PUT', '/1.0/containers/%s/state'
                                          % container,
                                          json.dumps(action))

    def container_suspend(self, container, timeout):
        action = {'action': 'freeze', 'timeout': timeout}
        return self.connection.get_object('PUT', '/1.0/containers/%s/state'
                                          % container,
                                          json.dumps(action))

    def container_resume(self, container, timeout):
        action = {'action': 'unfreeze', 'timeout': timeout}
        return self.connection.get_object('PUT', '/1.0/containers/%s/state'
                                          % container,
                                          json.dumps(action))

    def container_reboot(self, container, timeout):
        action = {'action': 'restart', 'timeout': timeout}
        return self.connection.get_object('PUT', '/1.0/containers/%s/state'
                                          % container,
                                          json.dumps(action))

    def container_destroy(self, container):
        return self.connection.get_object('DELETE', '/1.0/containers/%s'
                                          % container)

    def get_container_log(self, container):
        (state, data) = self.connection.get_object('GET', '/1.0/containers/%s?log=true'
                                                            % container)
        return data['metadata']['log']

    # file operations
    def get_container_file(self, container, filename):
        return self.connection.get_raw('GET', '/1.0/containers/%s/files?path=%s'
                                          % (container, filename))

    def container_publish(self, container):
        return self.connection.get_object('POST', '/1.0/images',
                                          json.dumps(container))

    # misc operations
    def run_command(self, container, args, interactive, web_sockets, env):
        env = env or {}
        data = {'command': args,
                'interactive': interactive,
                'wait-for-websocket': web_sockets,
                'environment': env}
        return self.connection.get_object('POST', '/1.0/containers/%s/exec'
                                          % container, json.dumps(data))

    # snapshots
    def snapshot_list(self, container):
        (state, data) = self.connection.get_object('GET',
                                            '/1.0/containers/%s/snapshots'
                                            % container)
        return [snapshot.split('/1.0/containers/%s/snapshots/%s/'
                                % (container, container))[-1] \
                for snapshot in data['metadata']]

    def snapshot_create(self, container, config):
        return self.connection.get_object('POST',
                                                '/1.0/containers/%s/snapshots'
                                                % container,
                                                json.dumps(config))

    def snapshot_info(self, container, snapshot):
        return self.conncetion.get_object('GET',
                                          '/1.0/containers/%s/snapshsots/%s'
                                          % (container, snapshot))

    def snapshot_rename(self, container, snapshot, config):
        return self.connection.get_object('POST',
                                          '/1.0/containers/%s/snapshots/%s'
                                          % (container, snapshot),
                                          json.dumps(config))

    def snapshot_delete(self, container, snapshot):
        return self.connection.get_object('DELETE',
                                          '/1.0/containers/%s/snapshots/%s'
                                          % (container, snapshot))



# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
import unittest

from pylxd import api
from pylxd import connection

import fake_api

class LXDUnitTestImage(unittest.TestCase):
    def setUp(self):
        super(LXDUnitTestImage, self).setUp()
        self.lxd = api.API()

    def test_list_images(self):
        with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', fake_api.fake_image_list())
            self.assertEqual(1, len(self.lxd.image_list()))

    def test_get_image_defined_fail(self):
        with mock.patch.object(connection.LXDConnection, 'get_status') as ms:
            ms.return_value = False
            self.assertFalse(self.lxd.image_defined('test-image'))

    def test_get_image_defined(self):
        with mock.patch.object(connection.LXDConnection, 'get_status') as ms:
            ms.return_value = True
            self.assertFalse(self.lxd.image_defined('test-image'))

    def test_get_image_info(self):
        with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', fake_api.fake_image_info())
            self.assertIsInstance(self.lxd.image_info('04aac4257341'), dict)

    def test_image_upload_date(self):
        with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', fake_api.fake_image_info())
            self.assertEqual('2015-06-30 09:10:53', 
                            self.lxd.image_upload_date('04aac4257341', data=None))

    def test_image_create_date(self):
        with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', fake_api.fake_image_info())
            self.assertEqual('Unknown',
                            self.lxd.image_create_date('04aac4257341', data=None))

    def test_image_expire_date(self):
        with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', fake_api.fake_image_info())
            self.assertEqual('Unknown',
                            self.lxd.image_expire_date('04aac4257341', data=None))

    def test_image_delete(self):
        with mock.patch.object(connection.LXDConnection, 'get_status') as ms:
            ms.return_value = True
            self.assertTrue(self.lxd.image_delete('fake'))

    def test_image_update(self):
        with mock.patch.object(connection.LXDConnection, 'get_status') as ms:
            ms.return_value = True
            self.assertTrue(self.lxd.image_update('fake'))

    def test_image_rename(self):
        with mock.patch.object(connection.LXDConnection, 'get_status') as ms:
            ms.return_value = True
            self.assertTrue(self.lxd.image_rename('fake'))


# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

def fake_image_list_empty():
    return {
        "type": "sync",
        "status": "Success",
        "status_code": 200,
        "metadata": []
    }


def fake_image_list():
    return {
        "type": "sync",
        "status": "Success",
        "status_code": 200,
        "metadata": ['chuck']
    }


def fake_image_info():
    return {
        "type": "sync",
        "status": "Success",
        "status_code": 200,
        "metadata": {
            "aliases": [
                {
                    "target": "ubuntu",
                    "description": "ubuntu"
                }
            ],
                "architecture": 2,
                "fingerprint": "04aac4257341478b49c25d22cea8a6ce0489dc6c42d835367945e7596368a37f",
                "filename": "",
                "properties": {},
                "public": 0,
                "size": 67043148,
                "created_at": 0,
                "expires_at": 0,
                "uploaded_at": 1435669853
        }
    }

def fake_alias_list():
    return 	{
		"type": "sync",
		"status": "Success",
		"status_code": 200,
		"metadata": []
	}



import exceptions

class ContainerUnDefined(Exception):
    pass

class UntrustedHost(Exception):
    pass

class ContainerProfileCreateFail(Exception):
    pass

class ContainerProfileDeleteFail(Exception):
    pass

class ImageInvalidSize(Exception):
    pass

class APIError(Exception):
    def __init__(self, error, status_code):
        msg = 'Error %s - %s.' % (status_code, error)
        Exception.__init__(self, msg)
        self.status_code = status_code
        self.error = error



# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
import unittest

from pylxd import api
from pylxd import connection

class LXDUnitTestHost(unittest.TestCase):
    def setUp(self):
        super(LXDUnitTestHost, self).setUp()
        self.lxd = api.API()

        self.fake_host = {
		        "type": "sync",
		        "status": "Success",
		        "status_code": 200,
		        "metadata": {
			        "api_compat": 1,
			        "auth": "trusted",
			        "config": {},
			        "environment": {
				        "backing_fs": "ext4",
				        "driver": "lxc",
				        "kernel_version": "3.19.0-22-generic",
				        "lxc_version": "1.1.2",
				        "lxd_version": "0.12"
			        }
		        }
	    }

    @mock.patch.object(api.API, 'host_ping')
    def test_get_host_ping(self, mock_api):
        mock_api.return_value = True
        self.assertTrue(self.lxd.host_ping())

    @mock.patch.object(api.API, 'host_ping')
    def test_get_host_ping_fail(self, mock_api):
        mock_api.return_value = False
        self.assertFalse(self.lxd.host_ping())

    @mock.patch.object(api.API, 'host_info')
    def test_get_host_info(self, mock_api):
        mock_api.return_value = self.fake_host
        data = self.lxd.host_info()
        self.assertIsInstance(data, dict)

    def test_get_lxd_api_compat(self):
        with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', self.fake_host)
            data = self.lxd.get_lxd_api_compat(data=None)
            self.assertEqual(data, 1)

    def test_get_lxd_host_trusted(self):
         with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', self.fake_host)
            self.assertTrue(self.lxd.get_lxd_host_trust(data=None))

    def test_get_lxd_host_backing_fs(self):
         with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', self.fake_host)
            self.assertEqual('ext4', self.lxd.get_lxd_backing_fs(data=None))

    def test_get_lxd_driver(self):
        with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', self.fake_host)
            self.assertEqual('lxc', self.lxd.get_lxd_driver(data=None))

    def test_get_lxc_version(self):
        with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', self.fake_host)
            self.assertEqual('1.1.2', self.lxd.get_lxc_version(data=None))

    def test_get_lxd_version(self):
        with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', self.fake_host)
            self.assertEqual(0.12, self.lxd.get_lxd_version(data=None))

    def test_get_kernel_version(self):
        with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', self.fake_host)
            self.assertEqual('3.19.0-22-generic', self.lxd.get_kernel_version(data=None))


from . import exceptions


def wait_for_container(name, timeout):
    pass

def block_container():
    pass

def get_lxd_error(state, data):
    status_code = data.get('error_code')
    error = data.get('error')
    raise exceptions.APIError(error, status_code)



#!/usr/bin/python
#
# api_test.py: Test/demo of the python3-lxc API
#
# (C) Copyright Canonical Ltd. 2015
#
# Authors:
# Chuck Short <zulcss@ubuntu.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
# USA

import uuid
import sys
import subprocess
import time

# Let's pick a random name, avoiding clashes
CONTAINER_NAME = str(uuid.uuid1())

from pylxd import api

lxd = api.API()
try:
    lxd.container_defined(CONTAINER_NAME)
except Exception as e:
    print "Container doesnt exist: %s" % e

config = {'name': CONTAINER_NAME,
          'source': {'type': 'none'}}
lxd.container_init(config)
if lxd.container_defined(CONTAINER_NAME):
    print "Container is running"
else:
    print "Whoops!"
containers = lxd.container_list()
for x in containers:
    lxd.container_destroy(x)
